@@ -53,15 +53,9 @@ static PHP_NAMED_FUNCTION(zif_zip_entry_close);
 #endif
 #endif
 
-#if PHP_VERSION_ID < 50400
-#define ARG_PATH "s"
-#define KEY_ARG_DC
-#define KEY_ARG_CC
-#else
 #define ARG_PATH "p"
 #define KEY_ARG_DC , const zend_literal *key
 #define KEY_ARG_CC , key
-#endif
 
 #if PHP_VERSION_ID < 50500
 #define TYPE_ARG_DC
@@ -152,8 +146,6 @@ static char * php_zip_make_relative_path(char *path, int path_len) /* {{{ */
 }
 /* }}} */
 
-#ifdef PHP_ZIP_USE_OO
-
 #if PHP_VERSION_ID < 50600
 # define CWD_STATE_ALLOC(l) malloc(l)
 # define CWD_STATE_FREE(s)  free(s)
@@ -227,17 +219,6 @@ static int php_zip_extract_file(struct zip * za, char *dest, char *file, int fil
 	/* let see if the path already exists */
 	if (php_stream_stat_path_ex(file_dirname_fullpath, PHP_STREAM_URL_STAT_QUIET, &ssb, NULL) < 0) {
 
-#if defined(PHP_WIN32) && (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION == 1)
-		char *e;
-		e = file_dirname_fullpath;
-		while (*e) {
-			   if (*e == '/') {
-					   *e = DEFAULT_SLASH;
-			   }
-			   e++;
-		}
-#endif
-
 		ret = php_stream_mkdir(file_dirname_fullpath, 0777,  PHP_STREAM_MKDIR_RECURSIVE|REPORT_ERRORS, NULL);
 		if (!ret) {
 			efree(file_dirname_fullpath);
@@ -288,12 +269,7 @@ static int php_zip_extract_file(struct zip * za, char *dest, char *file, int fil
 		goto done;
 	}
 
-#if PHP_API_VERSION < 20100412
-	stream = php_stream_open_wrapper(fullpath, "w+b", REPORT_ERRORS|ENFORCE_SAFE_MODE, NULL);
-#else
 	stream = php_stream_open_wrapper(fullpath, "w+b", REPORT_ERRORS, NULL);
-#endif
-
 	if (stream == NULL) {
 		n = -1;
 		zip_fclose(zf);
@@ -764,8 +740,6 @@ int php_zip_pcre(char *regexp, int regexp_len, char *path, int path_len, zval *r
 }
 /* }}} */
 
-#endif
-
 /* {{{ arginfo */
 ZEND_BEGIN_ARG_INFO_EX(arginfo_zip_open, 0, 0, 1)
 	ZEND_ARG_INFO(0, filename)
@@ -823,16 +797,11 @@ static const zend_function_entry zip_functions[] = {
 	PHP_FE(zip_entry_name,		arginfo_zip_entry_name)
 	PHP_FE(zip_entry_compressedsize,		arginfo_zip_entry_compressedsize)
 	PHP_FE(zip_entry_compressionmethod,		arginfo_zip_entry_compressionmethod)
-#ifdef  PHP_FE_END
 	PHP_FE_END
-#else
-	{NULL,NULL,NULL}
-#endif
 };
 /* }}} */
 
 /* {{{ ZE2 OO definitions */
-#ifdef PHP_ZIP_USE_OO
 static zend_class_entry *zip_class_entry;
 static zend_object_handlers zip_object_handlers;
 
@@ -849,10 +818,8 @@ typedef struct _zip_prop_handler {
 
 	int type;
 } zip_prop_handler;
-#endif
 /* }}} */
 
-#ifdef PHP_ZIP_USE_OO
 static void php_zip_register_prop_handler(HashTable *prop_handler, char *name, zip_read_int_t read_int_func, zip_read_const_char_t read_char_func, zip_read_const_char_from_ze_t read_char_from_obj_func, int rettype TSRMLS_DC) /* {{{ */
 {
 	zip_prop_handler hnd;
@@ -934,21 +901,18 @@ static zval **php_zip_get_property_ptr_ptr(zval *object, zval *member TYPE_ARG_D
 		zval_copy_ctor(&tmp_member);
 		convert_to_string(&tmp_member);
 		member = &tmp_member;
-#if PHP_VERSION_ID >= 50400
 		key = NULL;
-#endif
 	}
 
 	ret = FAILURE;
 	obj = (ze_zip_object *)zend_objects_get_address(object TSRMLS_CC);
 
 	if (obj->prop_handler != NULL) {
-#if PHP_VERSION_ID >= 50400
 		if (key) {
 			ret = zend_hash_quick_find(obj->prop_handler, Z_STRVAL_P(member), Z_STRLEN_P(member)+1, key->hash_value, (void **) &hnd);
-		} else
-#endif
+		} else {
 			ret = zend_hash_find(obj->prop_handler, Z_STRVAL_P(member), Z_STRLEN_P(member)+1, (void **) &hnd);
+		}
 	}
 
 
@@ -978,21 +942,18 @@ static zval* php_zip_read_property(zval *object, zval *member, int type KEY_ARG_
 		zval_copy_ctor(&tmp_member);
 		convert_to_string(&tmp_member);
 		member = &tmp_member;
-#if PHP_VERSION_ID >= 50400
 		key = NULL;
-#endif
 	}
 
 	ret = FAILURE;
 	obj = (ze_zip_object *)zend_objects_get_address(object TSRMLS_CC);
 
 	if (obj->prop_handler != NULL) {
-#if PHP_VERSION_ID >= 50400
 		if (key) {
 			ret = zend_hash_quick_find(obj->prop_handler, Z_STRVAL_P(member), Z_STRLEN_P(member)+1, key->hash_value, (void **) &hnd);
-		} else
-#endif
+		} else {
 			ret = zend_hash_find(obj->prop_handler, Z_STRVAL_P(member), Z_STRLEN_P(member)+1, (void **) &hnd);
+		}
 	}
 
 	if (ret == SUCCESS) {
@@ -1028,21 +989,18 @@ static int php_zip_has_property(zval *object, zval *member, int type KEY_ARG_DC
 		zval_copy_ctor(&tmp_member);
 		convert_to_string(&tmp_member);
 		member = &tmp_member;
-#if PHP_VERSION_ID >= 50400
 		key = NULL;
-#endif
 	}
 
 	ret = FAILURE;
 	obj = (ze_zip_object *)zend_objects_get_address(object TSRMLS_CC);
 
 	if (obj->prop_handler != NULL) {
-#if PHP_VERSION_ID >= 50400
 		if (key) {
 			ret = zend_hash_quick_find(obj->prop_handler, Z_STRVAL_P(member), Z_STRLEN_P(member)+1, key->hash_value, (void **) &hnd);
-		} else
-#endif
+		} else {
 			ret = zend_hash_find(obj->prop_handler, Z_STRVAL_P(member), Z_STRLEN_P(member)+1, (void **) &hnd);
+		}
 	}
 
 	if (ret == SUCCESS) {
@@ -1078,7 +1036,6 @@ static int php_zip_has_property(zval *object, zval *member, int type KEY_ARG_DC
 }
 /* }}} */
 
-#if PHP_VERSION_ID >= 50400
 static HashTable *php_zip_get_gc(zval *object, zval ***gc_data, int *gc_data_count TSRMLS_DC) /* {{{ */
 {
 	*gc_data = NULL;
@@ -1086,7 +1043,6 @@ static HashTable *php_zip_get_gc(zval *object, zval ***gc_data, int *gc_data_cou
 	return zend_std_get_properties(object TSRMLS_CC);
 }
 /* }}} */
-#endif
 
 static HashTable *php_zip_get_properties(zval *object TSRMLS_DC)/* {{{ */
 {
@@ -1122,6 +1078,34 @@ static HashTable *php_zip_get_properties(zval *object TSRMLS_DC)/* {{{ */
 }
 /* }}} */
 
+#ifdef HAVE_PROGRESS_CALLBACK
+static void _php_zip_progress_callback_free(void *ptr)
+{
+	ze_zip_object *obj = ptr;
+	TSRMLS_FETCH();
+
+	if (obj->progress_callback) {
+		zval_dtor(obj->progress_callback);
+		FREE_ZVAL(obj->progress_callback);
+		obj->progress_callback = NULL;
+	}
+}
+#endif
+
+#ifdef HAVE_CANCEL_CALLBACK
+static void _php_zip_cancel_callback_free(void *ptr)
+{
+	ze_zip_object *obj = ptr;
+	TSRMLS_FETCH();
+
+	if (obj->cancel_callback) {
+		zval_dtor(obj->cancel_callback);
+		FREE_ZVAL(obj->cancel_callback);
+		obj->cancel_callback = NULL;
+	}
+}
+#endif
+
 static void php_zip_object_free_storage(void *object TSRMLS_DC) /* {{{ */
 {
 	ze_zip_object * intern = (ze_zip_object *) object;
@@ -1150,18 +1134,16 @@ static void php_zip_object_free_storage(void *object TSRMLS_DC) /* {{{ */
 
 	intern->za = NULL;
 
-#if (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION == 1 && PHP_RELEASE_VERSION > 2) || (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION > 1) || (PHP_MAJOR_VERSION > 5)
 	zend_object_std_dtor(&intern->zo TSRMLS_CC);
-#else
-	if (intern->zo.guards) {
-		zend_hash_destroy(intern->zo.guards);
-		FREE_HASHTABLE(intern->zo.guards);
-	}
 
-	if (intern->zo.properties) {
-		zend_hash_destroy(intern->zo.properties);
-		FREE_HASHTABLE(intern->zo.properties);
-	}
+#ifdef HAVE_PROGRESS_CALLBACK
+	/* if not properly called by libzip */
+	_php_zip_progress_callback_free(intern);
+#endif
+
+#ifdef HAVE_CANCEL_CALLBACK
+	/* if not properly called by libzip */
+	_php_zip_cancel_callback_free(intern);
 #endif
 
 	if (intern->filename) {
@@ -1173,36 +1155,17 @@ static void php_zip_object_free_storage(void *object TSRMLS_DC) /* {{{ */
 
 static zend_object_value php_zip_object_new(zend_class_entry *class_type TSRMLS_DC) /* {{{ */
 {
-#if PHP_VERSION_ID < 50400
-	zval *tmp;
-#endif
 	ze_zip_object *intern;
 	zend_object_value retval;
 
 	intern = emalloc(sizeof(ze_zip_object));
+	memset(intern, 0, sizeof(ze_zip_object));
 	memset(&intern->zo, 0, sizeof(zend_object));
 
-	intern->za = NULL;
-	intern->buffers = NULL;
-	intern->filename = NULL;
-	intern->buffers_cnt = 0;
 	intern->prop_handler = &zip_prop_handlers;
 
-#if ((PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION > 1) || (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION == 1 && PHP_RELEASE_VERSION > 2))
 	zend_object_std_init(&intern->zo, class_type TSRMLS_CC);
-#else
-	ALLOC_HASHTABLE(intern->zo.properties);
-  	zend_hash_init(intern->zo.properties, 0, NULL, ZVAL_PTR_DTOR, 0);
-	intern->zo.ce = class_type;
-#endif
-
-
-#if PHP_VERSION_ID < 50400
-	zend_hash_copy(intern->zo.properties, &class_type->default_properties, (copy_ctor_func_t) zval_add_ref,
-					(void *) &tmp, sizeof(zval *));
-#else
 	object_properties_init(&intern->zo, class_type);
-#endif
 	retval.handle = zend_objects_store_put(intern,
 						NULL,
 						(zend_objects_free_object_storage_t) php_zip_object_free_storage,
@@ -1213,7 +1176,6 @@ static zend_object_value php_zip_object_new(zend_class_entry *class_type TSRMLS_
 	return retval;
 }
 /* }}} */
-#endif
 
 /* {{{ Resource dtors */
 
@@ -1549,7 +1511,6 @@ static PHP_NAMED_FUNCTION(zif_zip_entry_compressionmethod)
 }
 /* }}} */
 
-#ifdef PHP_ZIP_USE_OO
 /* {{{ proto mixed ZipArchive::open(string source [, int flags])
 Create new zip using source uri for output, return TRUE on success or the error code */
 static ZIPARCHIVE_METHOD(open)
@@ -1774,11 +1735,12 @@ static ZIPARCHIVE_METHOD(addEmptyDir)
 	if (idx >= 0) {
 		RETVAL_FALSE;
 	} else {
-		if (zip_add_dir(intern, (const char *)s) == -1) {
+		if (zip_dir_add(intern, (const char *)s, 0) == -1) {
 			RETVAL_FALSE;
+		} else {
+			zip_error_clear(intern);
+			RETVAL_TRUE;
 		}
-		zip_error_clear(intern);
-		RETVAL_TRUE;
 	}
 
 	if (s != dirname) {
@@ -3133,6 +3095,142 @@ static ZIPARCHIVE_METHOD(getStream)
 }
 /* }}} */
 
+#ifdef HAVE_PROGRESS_CALLBACK
+static void _php_zip_progress_callback(zip_t *arch, double state, void *ptr)
+{
+	zval *cb_state;
+	zval **cb_args[1];
+	zval *cb_retval;
+	ze_zip_object *obj = ptr;
+	TSRMLS_FETCH();
+
+	MAKE_STD_ZVAL(cb_state);
+	ZVAL_DOUBLE(cb_state, state);
+	cb_args[0] = &cb_state;
+	if (call_user_function_ex(EG(function_table), NULL, obj->progress_callback, &cb_retval, 1, cb_args, 0, NULL TSRMLS_CC) == SUCCESS && cb_retval) {
+		zval_ptr_dtor(&cb_retval);
+	}
+}
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ziparchive_registerprogresscallback, 0, 0, 2)
+	ZEND_ARG_INFO(0, rate)
+	ZEND_ARG_INFO(0, callback)
+ZEND_END_ARG_INFO()
+
+/* {{{ proto bool ZipArchive::registerProgressCallback(double rate, callable callback)
+register a progression callback: void callback(double state); */
+static ZIPARCHIVE_METHOD(registerProgressCallback)
+{
+	struct zip *intern;
+	zval *self = getThis();
+	double rate;
+	zval *callback;
+	ze_zip_object *obj;
+	char *callback_name;
+
+	if (!self) {
+		RETURN_FALSE;
+	}
+
+	ZIP_FROM_OBJECT(intern, self);
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "dz", &rate, &callback) == FAILURE) {
+		return;
+	}
+
+	/* callable? */
+	if (!zend_is_callable(callback, 0, &callback_name TSRMLS_CC)) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid callback '%s'", callback_name);
+		efree(callback_name);
+		RETURN_FALSE;
+	}
+	efree(callback_name);
+
+	obj = (ze_zip_object*) zend_object_store_get_object(self TSRMLS_CC);
+
+	/* free if called twice */
+	_php_zip_progress_callback_free(obj);
+
+	/* register */
+	ALLOC_ZVAL(obj->progress_callback);
+	*obj->progress_callback = *callback;
+	zval_copy_ctor(obj->progress_callback);
+	if (zip_register_progress_callback_with_state(intern, rate, _php_zip_progress_callback, _php_zip_progress_callback_free, obj)) {
+		RETURN_FALSE;
+	}
+
+	RETURN_TRUE;
+}
+/* }}} */
+#endif
+
+#ifdef HAVE_CANCEL_CALLBACK
+static int _php_zip_cancel_callback(zip_t *arch, void *ptr)
+{
+	zval *cb_retval;
+	int retval = 0;
+	ze_zip_object *obj = ptr;
+	TSRMLS_FETCH();
+
+	if (call_user_function_ex(EG(function_table), NULL, obj->cancel_callback, &cb_retval, 0, NULL, 0, NULL TSRMLS_CC) == SUCCESS && cb_retval) {
+		convert_to_long_ex(&cb_retval);
+		retval = Z_LVAL_P(cb_retval);
+		zval_ptr_dtor(&cb_retval);
+	}
+
+	return retval;
+}
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_ziparchive_registercancelcallback, 0, 0, 1)
+	ZEND_ARG_INFO(0, callback)
+ZEND_END_ARG_INFO()
+
+/* {{{ proto bool ZipArchive::registerCancelCallback(callable callback)
+register a progression callback: int callback(double state); */
+static ZIPARCHIVE_METHOD(registerCancelCallback)
+{
+	struct zip *intern;
+	zval *self = getThis();
+	zval *callback;
+	ze_zip_object *obj;
+	char *callback_name;
+
+	if (!self) {
+		RETURN_FALSE;
+	}
+
+	ZIP_FROM_OBJECT(intern, self);
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &callback) == FAILURE) {
+		return;
+	}
+
+	/* callable? */
+	if (!zend_is_callable(callback, 0, &callback_name TSRMLS_CC)) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid callback '%s'", callback_name);
+		efree(callback_name);
+		RETURN_FALSE;
+	}
+	efree(callback_name);
+
+	obj = (ze_zip_object*) zend_object_store_get_object(self TSRMLS_CC);
+
+	/* free if called twice */
+	_php_zip_cancel_callback_free(obj);
+
+	/* register */
+	ALLOC_ZVAL(obj->cancel_callback);
+	*obj->cancel_callback = *callback;
+	zval_copy_ctor(obj->cancel_callback);
+	if (zip_register_cancel_callback_with_state(intern, _php_zip_cancel_callback, _php_zip_cancel_callback_free, obj)) {
+		RETURN_FALSE;
+	}
+
+	RETURN_TRUE;
+}
+/* }}} */
+#endif
+
 /* {{{ arginfo */
 ZEND_BEGIN_ARG_INFO_EX(arginfo_ziparchive_open, 0, 0, 1)
 	ZEND_ARG_INFO(0, filename)
@@ -3371,25 +3469,27 @@ static const zend_function_entry zip_class_functions[] = {
 #ifdef HAVE_ENCRYPTION
 	ZIPARCHIVE_ME(setEncryptionName,		arginfo_ziparchive_setencryption_name, ZEND_ACC_PUBLIC)
 	ZIPARCHIVE_ME(setEncryptionIndex,		arginfo_ziparchive_setencryption_index, ZEND_ACC_PUBLIC)
+#endif
+#ifdef HAVE_PROGRESS_CALLBACK
+	ZIPARCHIVE_ME(registerProgressCallback,	arginfo_ziparchive_registerprogresscallback, ZEND_ACC_PUBLIC)
+#endif
+#ifdef HAVE_CANCEL_CALLBACK
+	ZIPARCHIVE_ME(registerCancelCallback,	arginfo_ziparchive_registercancelcallback, ZEND_ACC_PUBLIC)
 #endif
 	{NULL, NULL, NULL}
 };
 /* }}} */
-#endif
 
 /* {{{ PHP_MINIT_FUNCTION */
 static PHP_MINIT_FUNCTION(zip)
 {
-#ifdef PHP_ZIP_USE_OO
 	zend_class_entry ce;
 
 	memcpy(&zip_object_handlers, zend_get_std_object_handlers(), sizeof(zend_object_handlers));
 	zip_object_handlers.clone_obj		= NULL;
 	zip_object_handlers.get_property_ptr_ptr = php_zip_get_property_ptr_ptr;
 
-#if PHP_VERSION_ID >= 50400
 	zip_object_handlers.get_gc          = php_zip_get_gc;
-#endif
 	zip_object_handlers.get_properties = php_zip_get_properties;
 	zip_object_handlers.read_property	= php_zip_read_property;
 	zip_object_handlers.has_property	= php_zip_has_property;
@@ -3412,6 +3512,9 @@ static PHP_MINIT_FUNCTION(zip)
 	REGISTER_ZIP_CLASS_CONST_LONG("EXCL", ZIP_EXCL);
 	REGISTER_ZIP_CLASS_CONST_LONG("CHECKCONS", ZIP_CHECKCONS);
 	REGISTER_ZIP_CLASS_CONST_LONG("OVERWRITE", ZIP_OVERWRITE);
+#ifdef ZIP_RDONLY
+	REGISTER_ZIP_CLASS_CONST_LONG("RDONLY", ZIP_RDONLY);
+#endif
 
 	REGISTER_ZIP_CLASS_CONST_LONG("FL_NOCASE", ZIP_FL_NOCASE);
 	REGISTER_ZIP_CLASS_CONST_LONG("FL_NODIR", ZIP_FL_NODIR);
@@ -3547,7 +3650,6 @@ static PHP_MINIT_FUNCTION(zip)
 #endif
 
 	php_register_url_stream_wrapper("zip", &php_stream_zip_wrapper TSRMLS_CC);
-#endif /* ifdef PHP_ZIP_USE_OO */
 
 	le_zip_dir   = zend_register_list_destructors_ex(php_zip_free_dir,   NULL, le_zip_dir_name,   module_number);
 	le_zip_entry = zend_register_list_destructors_ex(php_zip_free_entry, NULL, le_zip_entry_name, module_number);
@@ -3560,10 +3662,9 @@ static PHP_MINIT_FUNCTION(zip)
  */
 static PHP_MSHUTDOWN_FUNCTION(zip)
 {
-#ifdef PHP_ZIP_USE_OO
 	zend_hash_destroy(&zip_prop_handlers);
 	php_unregister_url_stream_wrapper("zip" TSRMLS_CC);
-#endif
+
 	return SUCCESS;
 }
 /* }}} */
