@@ -37,27 +37,11 @@ extern zend_module_entry zip_module_entry;
 #define ZIP_OVERWRITE ZIP_TRUNCATE
 #endif
 
-#define PHP_ZIP_VERSION "1.16.1"
-
-#if ((PHP_MAJOR_VERSION >= 5 && PHP_MINOR_VERSION >= 2) || PHP_MAJOR_VERSION >= 6)
-# define PHP_ZIP_USE_OO 1
-#endif
-
-#ifndef  Z_SET_REFCOUNT_P
-# if PHP_MAJOR_VERSION < 6 && (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION < 3)
-#  define Z_SET_REFCOUNT_P(pz, rc)  pz->refcount = rc 
-#  define Z_UNSET_ISREF_P(pz) pz->is_ref = 0 
-# endif
-#endif
+#define PHP_ZIP_VERSION "1.17.1"
 
 /* {{{ ZIP_OPENBASEDIR_CHECKPATH(filename) */
-#if PHP_API_VERSION < 20100412
-# define ZIP_OPENBASEDIR_CHECKPATH(filename) \
-	(PG(safe_mode) && (!php_checkuid(filename, NULL, CHECKUID_CHECK_FILE_AND_DIR))) || php_check_open_basedir(filename TSRMLS_CC)
-#else
 #define ZIP_OPENBASEDIR_CHECKPATH(filename) \
 	php_check_open_basedir(filename TSRMLS_CC)
-#endif
 /* }}} */
 
 typedef struct _ze_zip_rsrc {
@@ -73,11 +57,9 @@ typedef struct _ze_zip_read_rsrc {
 	struct zip_stat sb;
 } zip_read_rsrc;
 
-#ifdef PHP_ZIP_USE_OO 
 #define ZIPARCHIVE_ME(name, arg_info, flags) {#name, c_ziparchive_ ##name, arg_info,(zend_uint) (sizeof(arg_info)/sizeof(struct _zend_arg_info)-1), flags },
 #define ZIPARCHIVE_METHOD(name)	ZEND_NAMED_FUNCTION(c_ziparchive_ ##name)
 
-
 /* Extends zend object */
 typedef struct _ze_zip_object {
 	zend_object zo;
@@ -87,6 +69,12 @@ typedef struct _ze_zip_object {
 	HashTable *prop_handler;
 	char *filename;
 	int filename_len;
+#ifdef HAVE_PROGRESS_CALLBACK
+	zval *progress_callback;
+#endif
+#ifdef HAVE_CANCEL_CALLBACK
+	zval *cancel_callback;
+#endif
 } ze_zip_object;
 
 #if PHP_VERSION_ID < 50600
@@ -97,7 +85,6 @@ php_stream *php_stream_zip_opener(php_stream_wrapper *wrapper, const char *path,
 php_stream *php_stream_zip_open(const char *filename, const char *path, const char *mode STREAMS_DC TSRMLS_DC);
 
 extern php_stream_wrapper php_stream_zip_wrapper;
-#endif
 
 #endif	/* PHP_ZIP_H */
 
