@@ -33,7 +33,7 @@ extern zend_module_entry zip_module_entry;
 #define ZIP_OVERWRITE ZIP_TRUNCATE
 #endif
 
-#define PHP_ZIP_VERSION "1.16.1"
+#define PHP_ZIP_VERSION "1.17.1"
 
 #define ZIP_OPENBASEDIR_CHECKPATH(filename) php_check_open_basedir(filename)
 
@@ -62,6 +62,12 @@ typedef struct _ze_zip_object {
 	int filename_len;
 	int buffers_cnt;
 	zend_object zo;
+#ifdef HAVE_PROGRESS_CALLBACK
+	zval progress_callback;
+#endif
+#ifdef HAVE_CANCEL_CALLBACK
+	zval cancel_callback;
+#endif
 } ze_zip_object;
 
 static inline ze_zip_object *php_zip_fetch_object(zend_object *obj) {
